/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors: Adrian Szyndela <adrian.s@samsung.com>
 *          Łukasz Stelmach <l.stelmach@samsung.com>
 */
/**
 * @file crash-stack-x86_64.c
 * @brief unwinding call stacks, functions specific for x86_64
 */
#include "crash-stack.h"
#include <sys/user.h>
#include <string.h>

struct user_regs_struct g_registers;	///< static storage for ptrace buffer for registers

void *_crash_stack_get_memory_for_ptrace_registers(size_t *size)
{
	if (NULL != size)
		*size = sizeof(g_registers);
	return &g_registers;
}

void *_get_place_for_register_value(const char *regname, int regnum)
{
	/* Function is not used for x86 anymore, make it dummy
	 * It is still called by generic __get_registers_core function
	 * The return value is checked for NULL in __get_registers_core.
	 */
	return NULL;
}

void _crash_stack_set_ptrace_registers(void *regbuf)
{
	/* Make it dummy as it not used for x86 anymore
	 * However it is still called by generic __get_registers_ptrace functions
	 */
}

void _crash_stack_print_regs(FILE* outputfile)
{
	fprintf(outputfile, "\nRegister Information\n");
#if defined(__x86_64__)
	fprintf(outputfile, "gs  = 0x%016llx, fs  = 0x%016llx\nes  = 0x%016llx, ds  = 0x%016llx\n",
		g_registers.gs,
		g_registers.fs,
		g_registers.es,
		g_registers.ds);
	fprintf(outputfile, "rdi = 0x%016llx, rsi = 0x%016llx\nrbp = 0x%016llx, rsp = 0x%016llx\n",
		g_registers.rdi,
		g_registers.rsi,
		g_registers.rbp,
		g_registers.rsp);
	fprintf(outputfile, "rax = 0x%016llx, rbx = 0x%016llx\nrcx = 0x%016llx, rdx = 0x%016llx\n",
		g_registers.rax,
		g_registers.rbx,
		g_registers.rcx,
		g_registers.rdx);
	fprintf(outputfile, "r8  = 0x%016llx, r9  = 0x%016llx\nr10 = 0x%016llx, r11 = 0x%016llx\n",
		g_registers.r8,
		g_registers.r9,
		g_registers.r10,
		g_registers.r11);
	fprintf(outputfile, "r12 = 0x%016llx, r13 = 0x%016llx\nr14 = 0x%016llx, r15 = 0x%016llx\n",
		g_registers.r12,
		g_registers.r13,
		g_registers.r14,
		g_registers.r15);
	fprintf(outputfile, "rip = 0x%016llx\n",
		g_registers.rip);
#else
	fprintf(outputfile, "gs  = 0x%08lx, fs  = 0x%08lx\nes  = 0x%08lx, ds  = 0x%08lx\n",
	       g_registers.xgs,
	       g_registers.xfs,
	       g_registers.xes,
	       g_registers.xds);
	fprintf(outputfile, "edi = 0x%08lx, esi = 0x%08lx\nebp = 0x%08lx, esp = 0x%08lx\n",
	       g_registers.edi,
	       g_registers.esi,
	       g_registers.ebp,
	       g_registers.esp);
	fprintf(outputfile, "eax = 0x%08lx, ebx = 0x%08lx\necx = 0x%08lx, edx = 0x%08lx\n",
	       g_registers.eax,
	       g_registers.ebx,
	       g_registers.ecx,
	       g_registers.edx);
	fprintf(outputfile, "eip = 0x%08lx\n",
	       g_registers.eip);
#endif
}
